@@ -3,17 +3,16 @@ from __future__ import annotations
 from dataclasses import asdict, is_dataclass
 from enum import auto, Enum
 from functools import wraps
-from typing import Any, Callable, cast, Optional, Type, TYPE_CHECKING, Union
+from typing import Any, Callable, cast, Optional, Union
 
 from pydantic import BaseModel, ValidationError
+from pydantic.dataclasses import dataclass as pydantic_dataclass, is_builtin_dataclass
 from pydantic.schema import model_schema
-from quart import current_app, request
+from quart import current_app, request, ResponseReturnValue as QuartResponseReturnValue
 from werkzeug.datastructures import Headers
 from werkzeug.exceptions import BadRequest
 
-if TYPE_CHECKING:
-    from pydantic.dataclasses import Dataclass
-
+from .typing import PydanticModel, ResponseReturnValue
 
 QUART_SCHEMA_REQUEST_ATTRIBUTE = "_quart_schema_request_schema"
 QUART_SCHEMA_RESPONSE_ATTRIBUTE = "_quart_schema_response_schemas"
@@ -40,7 +39,7 @@ class DataSource(Enum):
     JSON = auto()
 
 
-def validate_querystring(model_class: Union[Type[BaseModel], Type[Dataclass]]) -> Callable:
+def validate_querystring(model_class: PydanticModel) -> Callable:
     """Validate the querystring arguments.
 
     This ensures that the query string arguments can be converted to
@@ -48,10 +47,13 @@ def validate_querystring(model_class: Union[Type[BaseModel], Type[Dataclass]]) -
     is raised which by default results in a 400 response.
 
     Arguments:
-        model_class: The model to use, either a pydantic dataclass or
-            a class that inherits from pydantic's BaseModel. All the
-            fields must be optional.
+        model_class: The model to use, either a dataclass, pydantic
+            dataclass or a class that inherits from pydantic's
+            BaseModel. All the fields must be optional.
     """
+    if is_builtin_dataclass(model_class):
+        model_class = pydantic_dataclass(model_class)
+
     schema = model_schema(model_class)
 
     if len(schema.get("required", [])) != 0:
@@ -75,7 +77,7 @@ def validate_querystring(model_class: Union[Type[BaseModel], Type[Dataclass]]) -
 
 
 def validate_request(
-    model_class: Union[Type[BaseModel], Type[Dataclass]],
+    model_class: PydanticModel,
     *,
     source: DataSource = DataSource.JSON,
 ) -> Callable:
@@ -87,11 +89,15 @@ def validate_request(
     in a 400 response.
 
     Arguments:
-        model_class: The model to use, either a pydantic dataclass or
-            a class that inherits from pydantic's BaseModel.
+        model_class: The model to use, either a dataclass, pydantic
+            dataclass or a class that inherits from pydantic's
+            BaseModel. All the fields must be optional.
         source: The source of the data to validate (json or form
             encoded).
     """
+    if is_builtin_dataclass(model_class):
+        model_class = pydantic_dataclass(model_class)
+
     schema = model_schema(model_class)
     if source == DataSource.FORM and any(
         schema["properties"][field]["type"] == "object" for field in schema["properties"]
@@ -120,9 +126,7 @@ def validate_request(
     return decorator
 
 
-def validate_response(
-    model_class: Union[Type[BaseModel], Type[Dataclass]], status_code: int = 200
-) -> Callable:
+def validate_response(model_class: PydanticModel, status_code: int = 200) -> Callable:
     """Validate the response data.
 
     This ensures that the response is a either dictionary that the
@@ -133,13 +137,18 @@ def validate_response(
     Quart encodes as JSON.
 
     Arguments:
-        model_class: The model to use, either a pydantic dataclass or
-            a class that inherits from pydantic's BaseModel.
+        model_class: The model to use, either a dataclass, pydantic
+            dataclass or a class that inherits from pydantic's
+            BaseModel. All the fields must be optional.
         status_code: The status code this validation applies
             to. Defaults to 200.
     """
+    if is_builtin_dataclass(model_class):
+        model_class = pydantic_dataclass(model_class)
 
-    def decorator(func: Callable) -> Callable:
+    def decorator(
+        func: Callable[..., ResponseReturnValue]
+    ) -> Callable[..., QuartResponseReturnValue]:
         schemas = getattr(func, QUART_SCHEMA_RESPONSE_ATTRIBUTE, {})
         schemas[status_code] = model_class
         setattr(func, QUART_SCHEMA_RESPONSE_ATTRIBUTE, schemas)
@@ -169,6 +178,8 @@ def validate_response(
                         raise ResponseSchemaValidationError(error)
                 elif type(value) == model_class:
                     model_value = value
+                elif is_builtin_dataclass(value):
+                    model_value = model_class(**asdict(value))
                 else:
                     raise ResponseSchemaValidationError()
                 if is_dataclass(model_value):
